% script_ci_sim_cli.m
% Octave script for testing the command line program "ci_sim_cli.exe"
% "ci_sim_cli.exe" is a program conceived for simulation of the perception with a
%   cochlear implant system
% The CLI version (command line interface) provides the simulation with a
%   configuration prepared through the command line, wihch is appropriate for
%   easy generation of simulation for a wide range of conditions
% The most important option for this program is:
%   ci_sim_cli -h
% or:
%   ci_sim_cli --help
% ...which provides the help and syntax of the program
% The performance and the results should be similar to those of the graphical
%   user interface available at the web:
%   https://www.ugr.es/~atv/web_ci_SIM/es/ci_sim.htm
% or:
%   https://wpd.ugr.es/~sig.proc.audiology/cochlear-implant-simulation/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Script prepared by Angel de la Torre, University of Granada
%          (atv@ugr.es), 13/05/2024
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
clc;

% executable file
file_exe = '.\ci_sim_cli';

% we read the audio file in "wav" format
% however, the example is in "au" format, and therefore, before, au-file is
%  converted to wav-file:
file_audio = 'dniaa0101.au';        % name of the audio file in "au" format
file_content1 = 'number sequence (in Spanish): 8 2 9 8 3 5 1 1    (female speaker)'
file_content2 = '     ocho dos nueve ocho tres cinco uno uno'
[x,fs] = audioread(file_audio);     % reading the file
f_audio_in = 'audio_tmp.wav';       % name of the input audio file in "wav" format
audiowrite(f_audio_in,x,fs);        % writing the audio file in wav format

% Asking for help to the program:
%    (1) preparing the command with sprintf();
%    (2) showing it in the octave console with fprintf(); and
%    (3) sending the command with system()
Command = sprintf('%s --help',file_exe);
fprintf('\nSending the command to the operating system:\n   %s\n\n',Command)
system(Command);

% a simple test, only one simulation
f_audio_out = 'audio_tmp_cisim.wav'; % name of the output file
% We simulate a CI with 25 electrodes, providing 1800 pulses per second, all of
% them inserted at the tonotopic optimal position, a very low interaction among channels...
% the rest of parameters with default values:
% .\ci_sim_cli.exe -r 1800 --n_chan_ci 25 --n_inserted_ch 25 --n_on 25 -d 0.1 -i input.wav -o output.wav
Command = sprintf('%s -i %s -o %s -r 1800 --n_chan_ci 25 --n_inserted_ch 25 --n_on 25 -d 0.1',file_exe,f_audio_in,f_audio_out);
fprintf('\nSending the command to the OS:\n   %s\n',Command)
system(Command);
fprintf('Simulation finished\n');

% lisening to the simulated output
[x,fs] = audioread(f_audio_out);
fprintf('%s\n%s\n',file_content1,file_content2);
sound(x,fs)

% a more complex test: number of channels changing (in the CI system) from 1 to 40
% (the influence of the number of channels)
NCHAN=[1 2 3 4 6 8 10 12 15 18 21 24 27 30 35 40];
for idx=1:length(NCHAN)
    nchan=NCHAN(idx);
    f_out=sprintf('audio_tmp_nc_%d.wav',nchan);
    Command = sprintf('%s -i %s -o %s -r 1800 --n_chan_ci %d --n_inserted_ch %d --n_on %d -d 0.1',file_exe,f_audio_in,f_out,nchan,nchan,nchan);
    fprintf('\nSending the command to the OS:\n   %s\n',Command)
    system(Command);
    [x,fs] = audioread(f_out); sound(x,fs);
end


% another test: stimulation rate changing from 100 pps to 4000 pps, always 12 electrodes (default)
% (the influence of the stimulation rate)
S_RATE=[100 200 300 400 600 800 1000 1500 1800 2100 2500 3000 3500 4000];
for idx=1:length(S_RATE)
    s_rate=S_RATE(idx);
    f_out=sprintf('audio_tmp_sr_%d.wav',nchan);
    Command = sprintf('%s -i %s -o %s -r %d -d 0.1',file_exe,f_audio_in,f_out,s_rate);
    fprintf('\nSending the command to the OS:\n   %s\n',Command)
    system(Command);
    [x,fs] = audioread(f_out); sound(x,fs);
end


return;
